package tests::ReportGeneratorTest;

use strict;

use base qw/ Lire::Test::TestCase tests::TestCaseUsingOldConfigVars/;

use Lire::ReportSpec;
use Lire::FilterSpec;
use Lire::DlfSchema;
use Lire::ReportGenerator;
use Lire::ReportParser::ReportBuilder;
use File::Basename qw/dirname/;
use Cwd qw/realpath/;
use Carp;

sub new {
    my $self = shift()->SUPER::new( @_ );

    $self->{dir} = realpath( dirname( __FILE__ ));
    $self->setup_old_config_vars();
    $self;
}

sub set_up {
    my $self = shift->SUPER::set_up();

    # This is the time zone in which the report with which we compare
    # was created.
    $self->{saved_tz} = $ENV{TZ};
    $ENV{TZ} = 'EST';

    $self->{cfg}{lr_reports_path} = [ $self->{dir} . "/reports" ];
    $self->{cfg}{lr_filters_path} = [ $self->{dir} . "/filters" ];
    $self->{cfg}{lr_schemas_path} = [ $self->{dir} . "/schemas" ];
}

sub tear_down {
    my $self = shift->SUPER::tear_down();

    # Perl has the bright idea of warning when
    # one assigns undef to an environment variable ?!?
    $ENV{TZ} = defined $self->{saved_tz} ? $self->{saved_tz} : '';
}

sub test_generate_report {
    my $self = $_[0];

    my $parser = new Lire::ReportParser::ReportBuilder;
    my $report_e = $parser->parsefile( "$self->{dir}/data/test.xml" );

    my $gen = new Lire::ReportGenerator( "test",
                                         $self->{dir} . "/data/test.cfg",
                                         $self->{dir} . "/data/test.dlf",
                                       );
    my $report_t = $gen->generate_report;

    # The difference in these fields are normal
    $report_t->{date} = $report_e->{date};
    $report_t->generator( $report_e->generator );

    $self->assert_deep_equals( $report_e, $report_t );
}

1;
