package tests::functional::LrEnv2ConfTest;

use strict;

use base qw/Lire::Test::FunctionalTestCase/;

use Lire::Config::Build qw/ac_path ac_info/;
use Lire::Config::SpecParser;
use Lire::Config::Parser;

sub new {
    my $self = shift->SUPER::new( @_ );

    my $parser = new Lire::Config::SpecParser();
    $parser->merge_specifications_dir( ac_path('datadir', 'PACKAGE') . "/config-spec" );

    $self->{config_spec} = $parser->configspec;

    return $self;
}

sub tear_down {
    my $self = $_[0];
    $self->SUPER::tear_down;

    unlink( $self->{unlink_file} )
      if $self->{unlink_file};
}

sub test_lr_env2conf_home {
    my $self = $_[0];

    my $old_file = $self->homedir . "/.lire/etc/defaults";
    my $new_file = $self->homedir . "/.lire/config/defaults.xml";

    $self->check_conversion( $old_file, $new_file );
}

sub test_lr_env2conf_sysdir {
    my $self = $_[0];

    my $old_file = ac_path( 'sysconfdir', 'PACKAGE' ) . "/defaults.local";
    my $new_file = ac_path( 'sysconfdir', 'PACKAGE' ) . "/config/defaults.local.xml";

    $self->{unlink_file} = $new_file;
    $self->check_conversion( $old_file, $new_file );
}

sub check_conversion {
    my ( $self, $old_file, $new_file ) = @_;

    my $gzip = ac_info( 'PATHTOGZIP' );
    $self->create_test_file( $old_file, <<EOF );
LR_DEBUG=1
LR_GZIP=$gzip
LR_ARCHIVE=
LR_UNKNOWN=Test
EOF

    my $result = $self->lire_run( "lr_env2conf $old_file" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_matches( qr/saved 3 configuration variables in $new_file/,
                           $result->stdout );
    $self->assert( -f $new_file, "lr_env2conf didn't create $new_file" );

    my $e_cfg = $self->{config_spec}->instance;
    $e_cfg->get( 'lr_debug' )->set( 1 );
    $e_cfg->get( 'lr_archive' )->set( 0 );
    $e_cfg->get( 'gzip_path' )->set( $gzip );

    # Silence many undefined warnings because we do not have
    # complete configuration.
    local $SIG{__WARN__} = sub { $self->annotate( join "", @_ ) };
    my $parser = new Lire::Config::Parser( spec => $self->{config_spec} );
    my $cfg = $parser->load_config_file( $new_file );
    $self->assert_deep_equals( $e_cfg->as_value, $cfg->global->as_value );
}

1;
