package tests::functional::LrEnvironmentTest;

use strict;

use base qw/Lire::Test::FunctionalTestCase/;

use Lire::Config::Build qw/ac_path/;

sub test_lr_environment {
    my $self = $_[0];

    my $result = $self->lire_run( ac_path( 'libexecdir', 'PACKAGE') .
                                  "/lr_environment" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_matches( qr/lr_environment info started with no argument/,
                           $result->stderr );
    $self->assert_does_not_match( qr/ (warning|err|crit) /, $result->stderr );

    # Test that some variables which can't be set unless a job
    # identifier is set
    $self->assert_does_not_match( qr/lr_service=/, $result->stdout );

    # Test some known default config variables
    $self->assert_matches( qr/lr_test_uid='1'/, $result->stdout );
    $self->assert_matches( qr/lr_debug=''/,  $result->stdout );
    $self->assert_matches( qr/lr_user_level='normal'/, $result->stdout );

    # Test backward compatibility vars
    $self->assert_matches( qr/LR_USERLEVEL='normal'/, $result->stdout );
    $self->assert_matches( qr/export.*LR_USERLEVEL/, $result->stdout );
}

sub test_shell_env {
    my $self = $_[0];

    my $lr_env = ac_path( 'libexecdir', 'PACKAGE') . "/lr_environment";
    my $result = $self->lire_run( qq{ eval `$lr_env`; echo "lr_debug=\$lr_debug"x; echo "LR_USERLEVEL=\$lr_user_level"x; echo "lr_test_uid=\$lr_test_uid"x; echo "lr_user_level=\$lr_user_level"x } );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert_matches( qr/lr_test_uid=1x/, $result->stdout );
    $self->assert_matches( qr/lr_debug=x/,  $result->stdout );
    $self->assert_matches( qr/lr_user_level=normalx/, $result->stdout );

    # Test backward compatibility vars
    $self->assert_matches( qr/LR_USERLEVEL=normalx/,
                           $result->stdout );
}
1;
