package tests::functional::LrLog2ReportTest;

use strict;

use base qw/Lire::Test::FunctionalTestCase tests::functional::TestSchemaFixture/;
use Lire::Config::Build qw/ac_info/;
use Lire::ReportParser::ReportBuilder;
use File::Basename qw/dirname/;
use Cwd 'realpath';
use Carp;

sub new {
    my $self = shift->SUPER::new(@_);

    $self->tests::functional::TestSchemaFixture::init();

    $self->{testsdir} = realpath(dirname( __FILE__ ) . "/..");
    $self->{report_cfg} = "$self->{testsdir}/data/test.cfg";

    return $self;
}

sub set_up {
    my $self = $_[0];
    $self->SUPER::set_up();

    $self->set_up_test_schemas();
    $self->set_up_test_specs();
    $self->set_up_test2dlf();
    $self->set_up_TestDlfConverter();
}

sub tear_down {
    my $self = $_[0];
    $self->SUPER::tear_down();
    $self->tests::functional::TestSchemaFixture::tear_down();
}

sub test_version {
    my $self = $_[0];

    my $result = $self->lire_run( "lr_log2report -v" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert( !$result->stdout, "stdout should be empty" );
    $self->assert_does_not_match( qr/ (crit|err|warning) / , $result->stderr );
    my $VERSION = ac_info( "VERSION" );
    $self->assert_matches( qr/shipped with Lire $VERSION/, $result->stderr );
}

sub test_old_custom_converter_report {
    my $self = $_[0];

    my $result = $self->lire_run( "lr_log2report -c $self->{report_cfg} -o xml test < /etc/passwd > report.xml" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert( !$result->stdout, "stdout should be empty" );
    $self->assert_does_not_match( qr/ (crit|err|warning) / , $result->stderr );

    $self->assert( -f $self->rundir . "/report.xml",
                   "XML report wasn't created" );
    $self->assert( -s $self->rundir . "/report.xml",
                   "XML report is empty" );

    $self->check_xml_report( "$self->{testsdir}/data/test.xml",
                             $self->rundir . "/report.xml" );
}

sub test_old_custom_converter_alias_report {
    my $self = $_[0];

    my $result = $self->lire_run( "lr_log2report -c $self->{report_cfg} -o xml test_alias < /etc/passwd > report.xml" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert( !$result->stdout, "stdout should be empty" );
    $self->assert_does_not_match( qr/ (crit|err|warning) / , $result->stderr );

    $self->assert( -f $self->rundir . "/report.xml",
                   "XML report wasn't created" );
    $self->assert( -s $self->rundir . "/report.xml",
                   "XML report is empty" );

    $self->check_xml_report( "$self->{testsdir}/data/test.xml",
                             $self->rundir . "/report.xml" );
}

sub test_custom_converter_report {
    my $self = $_[0];

    my $result = $self->lire_run( "lr_log2report -c $self->{report_cfg} -o xml test_newapi < $self->{testsdir}/data/test.dlf > report.xml" );
    $self->annotate( $result->stderr );
    $self->assert_equals( 0, $result->status );
    $self->assert( !$result->stdout, "stdout should be empty" );
    $self->assert_does_not_match( qr/ (crit|err|warning) / , $result->stderr );

    $self->assert( -f $self->rundir . "/report.xml",
                   "XML report wasn't created" );
    $self->assert( -s $self->rundir . "/report.xml",
                   "XML report is empty" );

    $self->check_xml_report( "$self->{testsdir}/data/test.xml",
                             $self->rundir . "/report.xml" );
}

sub test_bad_service {
    my $self = $_[0];

    my $result = $self->lire_run( "lr_log2report no_such_service" );
    $self->annotate( $result->stderr );
    $self->assert_not_equals( 0, $result->status );
    $self->assert( !$result->stdout, "stdout should be empty" );
    $self->assert_matches( qr/isn't a known DLF converter/,
                           $result->stderr );
}

sub test_usage {
    my $self = $_[0];

    my $result = $self->lire_run( "lr_log2report www combined" );
    $self->annotate( $result->stderr );
    $self->assert_not_equals( 0, $result->status );
    $self->assert( !$result->stdout, "stdout should be empty" );
    $self->assert_matches( qr/Usage lr_log2report/,
                           $result->stderr );
}

sub check_xml_report {
    my ($self, $expected_report, $gen_report ) = @_;

    my $parser = new Lire::ReportParser::ReportBuilder;
    my $report_e = $parser->parsefile( $expected_report );
    my $report_t = $parser->parsefile( $gen_report  );

    # The difference in these fields are normal
    $report_t->{date} = $report_e->{date};
    $report_t->generator( $report_e->generator );

    $self->assert_deep_equals( $report_e, $report_t );
}
1;
