package tests::functional::TestSchemaFixture;

use strict;

use Lire::Config::Build qw/ac_info ac_path/;
use File::Basename qw/dirname/;
use File::Path qw/mkpath rmtree/;
use File::Copy;
use IO::Dir;
use Cwd 'realpath';
use Carp;

sub init {
    my $self = $_[0];

    $self->{_tsh_testsdir} = realpath(dirname( __FILE__ ) . "/..");
    $self->{_tsh_converter_init} =
      ac_path( 'sysconfdir', 'PACKAGE' ) . "/converters/test_newapi";
    $self->{_tsh_aliases_file} =
      ac_path( 'sysconfdir', 'PACKAGE' ) . "/service_aliases.cf";
    $self->{_tsh_address_file} =
      ac_path( 'sysconfdir', 'PACKAGE' ) . "/address.cf";

    $self->{_tsh_moduledir} =
      ac_info( 'LR_PERL5LIB' ) . "/tests/helpers";

    return $self;
}

sub set_up_test_schemas {
    my $self = $_[0];

    foreach my $s ( $self->_find_xml_specs( "$self->{_tsh_testsdir}/schemas" ))
    {
        $self->install_xml_spec( "schema", "test", $s );
    }
}

sub set_up_test_specs {
    my $self = $_[0];

    # This is the time zone in which the report with which we compare
    # was created.
    $self->{saved_tz} = $ENV{TZ};
    $ENV{TZ} = 'EST';

    foreach my $s ( $self->_find_xml_specs( "$self->{_tsh_testsdir}/filters/test" ))
    {
        $self->install_xml_spec( "filter", "test", $s );
    }

    foreach my $s ( $self->_find_xml_specs( "$self->{_tsh_testsdir}/reports/test" ))
    {
        $self->install_xml_spec( "report", "test", $s );
    }

    mkpath( [ $self->{_tsh_moduledir} ], 0, 0755 );
    copy ( "$self->{_tsh_testsdir}/helpers/TestDerivedAnalyzer.pm",
           "$self->{_tsh_moduledir}/TestDerivedAnalyzer.pm" );
    copy ( "$self->{_tsh_testsdir}/helpers/TestExtendedAnalyzer.pm",
           "$self->{_tsh_moduledir}/TestExtendedAnalyzer.pm" );
}

sub set_up_test2dlf {
    my $self = $_[0];

    my $perl = ac_info( 'PATHTOPERL' );
    my $perllib = ac_info( 'LR_PERL5LIB' );
    my $dlf_converter = ac_path( "libexecdir", "PACKAGE" ) . "/convertors/test2dlf";
    $self->create_test_file( $dlf_converter, <<SCRIPT );
#! $perl -w

use strict;

use lib '$perllib';

use Lire::Program qw/:dlf/;
use Lire::Utils qw/file_content/;

init_dlf_converter( 'test' );
print file_content( '$self->{_tsh_testsdir}/data/test.dlf' );
end_dlf_converter( 24, 24, 0 );
SCRIPT

    chmod( 0755, $dlf_converter) == 1
      or croak "failed to set execute permissions on $dlf_converter";

    rename $self->{_tsh_address_file}, "$self->{_tsh_address_file}.bak"
      or croak "failed to rename $self->{_tsh_address_file}";
    $self->create_test_file( $self->{_tsh_address_file}, "test test\n" );
    rename $self->{_tsh_aliases_file}, "$self->{_tsh_aliases_file}.bak"
      or croak "failed to rename $self->{_tsh_aliases_file}";
    $self->create_test_file( $self->{_tsh_aliases_file}, "test_alias test\n" );
}

sub set_up_TestDlfConverter {
    my $self = $_[0];

    $self->create_test_file( $self->{_tsh_converter_init}, <<EOF );
use tests::helpers::TestDlfConverter;

return new tests::helpers::TestDlfConverter();
EOF

    mkpath( [ $self->{_tsh_moduledir} ], 0, 0755 );
    copy ( "$self->{_tsh_testsdir}/helpers/TestDlfConverter.pm",
           "$self->{_tsh_moduledir}/TestDlfConverter.pm" );
}

sub _find_xml_specs {
    my ( $self, $dir ) = @_;


    my $dir_h = new IO::Dir( $dir )
      or croak "failed to opendir $dir: $!";
    return map { $dir . "/$_" } grep { /\.xml$/ } $dir_h->read();
}

sub tear_down {
    my $self = $_[0];

    if ( exists $self->{saved_tz} ) {
        # Perl has the bright idea of warning when
        # one assigns undef to an environment variable ?!?
        $ENV{TZ} = defined $self->{saved_tz} ? $self->{saved_tz} : '';
    }

    if ( -f "$self->{_tsh_address_file}.bak" ) {
        rename "$self->{_tsh_address_file}.bak", $self->{_tsh_address_file}
          or croak "failed to rename $self->{_tsh_address_file}";
        rename "$self->{_tsh_aliases_file}.bak",  $self->{_tsh_aliases_file}
          or croak "failed to rename $self->{_tsh_aliases_file}";
    }
    rmtree( [ $self->{_tsh_moduledir}], 0, 0 );
}

1;
